unit uCohort;

interface

uses SysUtils, Classes, Forms, ComCtrls;

const
  SRC_CLINIC    = 'c';    //'44';
  SRC_WARD      = 'w';    //'42';
  SRC_TEAM      = 't';    //'100.21';
  SRC_SPECIALTY = 's';    //'45.7';
  SRC_PROVIDER  = 'p';    //'200';
  SRC_REGISTRY  = 'r';    //'798.1';

type
  TCohort = class
  private
    FSrcList: TStringList;             // string = 'SrcType:IEN' with TSelectedSrc objects
  public
    constructor Create;
    destructor Destroy; override;
    procedure Assign(Source: TCohort);
    procedure Clear;
    procedure AddPatients(const ASrcID, AParam, AName: string);
    procedure Exclude(ADFN: Int64);
    function LastSourceType: Char;
    procedure LoadSelectedSources(Dest: TStrings);
    procedure LoadSourceValues(Dest: TStrings);
    procedure RemovePatients(const ASrcID: string);
    procedure SetFromValueList(AValueList: TStringList);
    procedure UpdateSoruceValues(AParam: string);
    function SourceListCaption: string;
    property SrcList: TStringList   read FSrcList;
  end;

  TSelectedSrc = class                           // stored in FSrcList
    Id: string;
    Name: string;
    Param: string;
  end;
  
var
  Cohort: TCohort = nil;

implementation

uses ORFn, rPatients;

const
  TX_PRE  = '-- ';
  TX_POST = ' --';

{ TCohort ---------------------------------------------------------------------------------- }

{ TCohort constructors, destructors, assign & clear }

constructor TCohort.Create;
begin
  FSrcList := TStringList.Create;
end;

destructor TCohort.Destroy;
begin
  Clear;
  FSrcList.Free;
  inherited;
end;

procedure TCohort.Assign(Source: TCohort);
var
  i: Integer;
  SrcSrc, DestSrc: TSelectedSrc;
begin
  Self.Clear;
  for i := 0 to Pred(Source.FSrcList.Count) do
  begin
    SrcSrc        := TSelectedSrc(Source.FSrcList.Objects[i]);
    DestSrc       := TSelectedSrc.Create;
    DestSrc.Id   := SrcSrc.Id;
    DestSrc.Name  := SrcSrc.Name;
    DestSrc.Param := SrcSrc.Param;
    Self.FSrcList.AddObject(Source.FSrcList[i], DestSrc);
  end;
end;

procedure TCohort.Clear;
var
  i: Integer;
  ASelectedSrc: TSelectedSrc;
begin
  for i := Pred(FSrcList.Count) downto 0 do
  begin
    ASelectedSrc := TSelectedSrc(FSrcList.Objects[i]);
    ASelectedSrc.Free;
  end;
  FSrcList.Clear;
end;

{ TCohort public methods }

procedure TCohort.AddPatients(const ASrcID, AParam, AName: string);
{ given a source ID (source type character:ien) load a list of patients
  optional parameters are date/time range for clinics, mode for registries }
var
  AnId: string;
  ASelectedSrc: TSelectedSrc;
begin
  AnId := Piece(ASrcID, ':', 2);
  if FSrcList.IndexOf(ASrcID) < 0 then
  begin
    ASelectedSrc       := TSelectedSrc.Create;
    ASelectedSrc.Id   := AnId;
    ASelectedSrc.Name  := AName;
    ASelectedSrc.Param := AParam;
    FSrcList.AddObject(ASrcID, ASelectedSrc);
  end;
end;

procedure TCohort.Exclude(ADFN: Int64);
{ creates a list of excluded patients (i.e., after sensitive patient check }
begin
  // TODO: exclude patients by creating a list of excluded DFN's
end;

function TCohort.LastSourceType: Char;
{ returns the last type of source used to select patients }
begin
  Result := '0';
  if FSrcList.Count > 0 then Result := CharAt(FSrcList[Pred(FSrcList.Count)], 1);
end;

procedure TCohort.LoadSelectedSources(Dest: TStrings);
var
  i: Integer;
  SrcTypeName: string;
  ASelectedSrc: TSelectedSrc;
begin
  for i := 0 to Pred(FSrcList.Count) do
  begin
    case CharAt(FSrcList[i], 1) of
    SRC_CLINIC    : SrcTypeName := 'Clinic';
    SRC_WARD      : SrcTypeName := 'Ward';
    SRC_TEAM      : SrcTypeName := 'Team';
    SRC_SPECIALTY : SrcTypeName := 'Specialty';
    SRC_PROVIDER  : SrcTypeName := 'Provider';
    SRC_REGISTRY  : SrcTypeName := 'Registry';
    end;
    ASelectedSrc := TSelectedSrc(FSrcList.Objects[i]);
    Dest.Add(FSrcList[i] + U + ASelectedSrc.Name + U + SrcTypeName + U + ASelectedSrc.Param);
  end;
end;

procedure TCohort.LoadSourceValues(Dest: TStrings);
{ given a TStrings (list box), add all the source ID's (source type:ien:param) }
var
  i: Integer;
  ASelectedSrc: TSelectedSrc;
begin
  Dest.Clear;
  for i := 0 to Pred(FSrcList.Count) do
  begin
    ASelectedSrc := TSelectedSrc(FSrcList.Objects[i]);
    Dest.Add(FSrcList[i] + ':' + ASelectedSrc.Param);
  end;
end;

procedure TCohort.RemovePatients(const ASrcID: string);
{ remove all patients linked to a specific SourceID }
var
  i: Integer;
  ASelectedSrc: TSelectedSrc;
begin
  i := FSrcList.IndexOf(ASrcID);
  ASelectedSrc := TSelectedSrc(FSrcList.Objects[i]);
  ASelectedSrc.Free;
  FSrcList.Delete(i);
end;

procedure TCohort.SetFromValueList(AValueList: TStringList);
var
  i: Integer;
  ASrcID, AParam, AName, AnIEN: string;
begin
  for i := 0 to Pred(AValueList.Count) do
  begin
    ASrcID := Pieces(AValueList[i], ':', 1, 2);
    AParam := Copy(AValueList[i], Length(ASrcID) + 2, Length(AValueList[i]));
    AnIEN  := Piece(ASrcID, ':', 2);
    case CharAt(ASrcID, 1) of
      SRC_CLINIC    : AName := ExternalName(AnIEN, 44);
      SRC_WARD      : AName := ExternalName(AnIEN, 42);
      SRC_TEAM      : AName := ExternalName(AnIEN, 100.21);
      SRC_SPECIALTY : AName := ExternalName(AnIEN, 45.7);
      SRC_PROVIDER  : AName := ExternalName(AnIEN, 200);
      SRC_REGISTRY  : AName := RegistryName(Piece(ASrcID, ':', 2));
    end; {case}
    AddPatients(ASrcID, AParam, AName);
  end;
end;

function TCohort.SourceListCaption: string;
{ returns caption text that represents the contents of the patient cohort }
var
  i: Integer;
  ASelectedSrc: TSelectedSrc;
begin
  Result := '';
  for i := 0 to Pred(FSrcList.Count) do
  begin
    if i > 0 then Result := Result + ' and';
    case CharAt(FSrcList[i], 1) of
    SRC_CLINIC    : Result := Result + ' Clinic: ';
    SRC_WARD      : Result := Result + ' Ward: ';
    SRC_TEAM      : Result := Result + ' Team: ';
    SRC_SPECIALTY : Result := Result + ' Specialty: ';
    SRC_PROVIDER  : Result := Result + ' Provider: ';
    SRC_REGISTRY  : Result := Result + ' ';
    end; {case}
    ASelectedSrc := TSelectedSrc(FSrcList.Objects[i]);
    Result := Result + ASelectedSrc.Name;
  end; {for i}
  if Result = '' then Result := '< select patient list >';
end;

procedure TCohort.UpdateSoruceValues(AParam: string);
var
  i: integer;
begin
  for i := 0 to Pred(FSrcList.Count) do
  begin
    case CharAt(FSrcList[i], 1) of
    SRC_CLINIC    :
      TSelectedSrc(FSrcList.Objects[i]).Param := AParam;
    end;
  end;
end;


end.
