unit uPtQuery;

interface

uses SysUtils, Classes, uReport;

type
  TPtQuery = class
  private
    FReport: TReport;
    FFinished: Boolean;
    FIterator: string;
    FRecordCount: Integer;
    FPatientCount: Integer;
    FScreenedCount: Integer;
    FCacheFirst: Integer;
    FCacheLast: Integer;
    FCache: TStringList;
    FColumns: TStringList;
    procedure ClearCache;
    function GetRowStrings(Index: Integer): TStringList;
    function GetRowID(Index: Integer): string;
    function GetCell(ACol, ARow: Integer): string;
    procedure SetColumnTypes;
  public
    constructor Create(AReport: TReport);
    destructor Destroy; override;
    procedure CacheRowData(First, Last: Integer);
    procedure Cancel;
    procedure ExportToFile(const AName: string);
    procedure Initiate;
    procedure Iterate;
    procedure SortByColumn(AColumn:Integer; Ascending: Boolean);
    property Cell[ACol, ARow: Integer]: string read GetCell;
    property Finished: Boolean read FFinished;
    property ScreenedCount: Integer read FScreenedCount;
    property PatientCount: Integer read FPatientCount;
    property RecordCount: Integer read FRecordCount;
    property RowID[Index: Integer]: string read GetRowID;
    property RowStrings[Index: Integer]: TStringList read GetRowStrings;
  end;

implementation

uses ORFn, rReport, rPatients;

const
  MAX_CACHE_ITEMS    = 400;
  MAX_QUERY_PATIENTS = 5;
  COL_TYPE_DATE      = 1;
  COL_TYPE_OTHER     = 0;

{ TPtQuery }

{ constructors, destructors }

constructor TPtQuery.Create(AReport: TReport);
var
  i: Integer;
begin
  FReport    := AReport;
  FFinished  := False;
  FCache     := TStringList.Create;
  FColumns   := TStringList.Create;
  FColumns.Add('RowItemID');
  for i := 0 to Pred(FReport.ColCount) do FColumns.Add(FReport.ColName[i]);
  SetColumnTypes;
  FCacheFirst := -1;
  FCacheLast  := -1;
end;

destructor TPtQuery.Destroy;
begin
  ClearCache;
  FCache.Free;
  FColumns.Free;
  inherited;
end;

{ private methods }

procedure TPtQuery.ClearCache;
var
  i: Integer;
begin
  for i := 0 to Pred(FCache.Count) do TStringList(FCache.Objects[i]).Free;
  FCache.Clear;
  FCacheFirst := -1;
  FCacheLast  := -1;
end;

function TPtQuery.GetCell(ACol, ARow: Integer): string;
var
  Offset: Integer;
  ColStrings: TStringList;
begin
  Offset := ARow - FCacheFirst;
  Result := '';
  if Offset >= FCache.Count then Exit;
  ColStrings := TStringList(FCache.Objects[OffSet]);
  if ACol >= ColStrings.Count then Exit;
  Result := ColStrings[ACol];
end;

function TPtQuery.GetRowID(Index: Integer): string;
var
  Offset: Integer;
begin
  Offset := Index - FCacheFirst;
  Result := '';
  if Offset >= FCache.Count then Exit;
  Result := FCache[Offset];
end;

function TPtQuery.GetRowStrings(Index: Integer): TStringList;
begin
  Result := TStringList(FCache.Objects[Index]);
end;

{ public methods }

procedure TPtQuery.CacheRowData(First, Last: Integer);
var
  i, NextInsert, ColNum: Integer;
  RowData: TStringList;
  OneRow: TStringList;
begin
  // if range is within cache range we don't need to update anything
  if (First >= FCacheFirst) and (Last <= FCacheLast) then Exit;
  // if range is outside of cache or a superset of cache, start over
  if (Last < Pred(FCacheFirst)) or (First > Succ(FCacheLast)) or       // outside
     ((First < FCacheFirst) and (Last > FCacheLast)) or                // superset
     (FCache.Count > MAX_CACHE_ITEMS)                                  // exceeded max
     then ClearCache;
  // if getting items immediately before cache range
  if (First < FCacheFirst) and (Last  >= FCacheFirst) then Last  := Pred(FCacheFirst);
  // if getting items immediately after cache range
  if (Last  > FCacheLast)  and (First <= FCacheLast)  then First := Succ(FCacheLast);
  RowData := TStringList.Create;
  LoadQueryData(RowData, First, Last);
  NextInsert := 0;
  OneRow := nil;
  for i := 0 to Pred(RowData.Count) do
  begin
    if RowData.Names[i] = 'RowItemID' then
    begin
      OneRow := TStringList.Create;
      if First > FCacheLast
        then FCache.AddObject(ValuePart(RowData[i]), OneRow)                  // append
        else FCache.InsertObject(NextInsert, ValuePart(RowData[i]), OneRow);  // prepend
      Inc(NextInsert);
    end else
    begin
      if OneRow = nil then Continue;
      ColNum := OneRow.Add(ValuePart(RowData[i]));
      if Integer(FColumns.Objects[ColNum+1]) = COL_TYPE_DATE
        then OneRow[ColNum] := FormatFMDateTime('mm/dd/yy hh:nn', MakeFMDateTime(OneRow[ColNum]));
    end;
  end;
  // reset FAllFirst & FAllLast indexes to reflect current FAllItems
  if FCacheFirst < 0     then FCacheFirst := First;
  if FCacheLast  < 0     then FCacheLast  := Last;
  if First < FCacheFirst then FCacheFirst := First;
  if Last  > FCacheLast  then FCacheLast  := Last;
end;

procedure TPtQuery.Cancel;
begin
  ClearQuery;
end;

procedure TPtQuery.ExportToFile(const AName: string);
const
  CACHE_INCREMENT = 30;
var
  ACol, ARow, LastCol, FirstCache, LastCache: Integer;
  ALine: string;
  OutFile: TextFile;
begin
  AssignFile(OutFile, AName);
  Rewrite(OutFile);
  LastCol := FColumns.Count - 1;
  FirstCache := 0;
  LastCache := CACHE_INCREMENT;
  CacheRowData(FirstCache, LastCache);
  ARow := 0;
  while ARow < FRecordCount do
  begin
    ALine := '';
    // skip column 0 since it is the internal identifier
    for ACol := 0 to LastCol do ALine := ALine + GetCell(ACol, ARow) + #9;
    Writeln(OutFile, ALine);
    Inc(ARow);
    if ARow > LastCache then
    begin
      FirstCache := ARow;
      LastCache := LowerOf(ARow + CACHE_INCREMENT, FRecordCount - 1);
      CacheRowData(FirstCache, LastCache);
    end;
  end;
  CloseFile(OutFile);
end;

procedure TPtQuery.Initiate;
var
  QueryStrings: TStringList;
begin
  FFinished      := False;
  FIterator      := '';
  FPatientCount  := 0;
  FRecordCount   := 0;
  FScreenedCount := 0;
  QueryStrings := TStringList.Create;
  FReport.QueryToStrings(QueryStrings);
  if QueryStrings.Count > 0 then FIterator := SetupQuery(QueryStrings);
  QueryStrings.Free;
  if FIterator = '' then FFinished := True;
end;

procedure TPtQuery.Iterate;
var
  x: string;
begin
  x := QueryIterate(FIterator);
  FIterator := Piece(x, U, 3);
  if FIterator = '' then FFinished := True;
  if Piece(x, U, 1) = '1'
    then Inc(FPatientCount)
    else Inc(FScreenedCount);
  Inc(FRecordCount, StrToIntDef(Piece(x, U, 2), 0));
end;

procedure TPtQuery.SortByColumn(AColumn:Integer; Ascending: Boolean);
begin
  ClearCache;
  if AColumn < FColumns.Count then SortByFieldName(FColumns[AColumn], Ascending);
  if (AColumn=FColumns.Count) and (FColumns.Count=1) then     
    SortByFieldName(FColumns[0], Ascending);
end;

procedure TPtQuery.SetColumnTypes;
var
  i: Integer;
  ColTypeList: TStringList;
begin
  ColTypeList := TStringList.Create;
  ColTypeList.Assign(FColumns);
  LoadColumnTypes(ColTypeList);
  for i := 0 to Pred(FColumns.Count) do
  begin
    if Piece(ColTypeList[i], U, 2) = 'D'
      then FColumns.Objects[i] := Pointer(COL_TYPE_DATE)
      else FColumns.Objects[i] := Pointer(COL_TYPE_OTHER);
  end;
  ColTypeList.Free;
end;

end.
